import json
import numpy as np
import copy
from  coords import rotationAboutAxis
from arbdmodel import Group, PointParticle, ParticleType

seqComplement = dict(A='T',G='C')
resnames = dict(A="ADE", G="GUA", C="CYT", T="THY")
for x,y in list(seqComplement.items()): seqComplement[y] = x

def stringToIntTuples(string, tupleLen, offset):
    l = string.split()
    # for i in range(0,len(l),tupleLen):
    #     ret.append( (int(x)-offset for x in l[i:i+tupleLen]) )
    ret = [ (int(x)-offset for x in l[i:i+tupleLen]) 
            for i in range(0,len(l),tupleLen) ]

    return ret

class CanonicalNucleotideFactory(Group):
    DefaultOrientation = rotationAboutAxis([0,0,1], 180)
    def __init__(self, prefix, seq):
        self.sequence = seq     # TODO: used?
        self.resname = resnames[seq]
        self.children = []

        ## idx, type,   
        d = np.loadtxt("%s.dat" % prefix, dtype='i4,S4,S4,f4,f4,f4,f4,f4')

        ids = d['f0']
        assert( np.all(ids == sorted(ids)) )

        names = d['f1']
        tns = d['f2']
        xs = d['f5']
        ys = d['f6']
        zs = d['f7']

        type_list = []
        for i in range(len(tns)):
            tn = tns[i]
            type_ = ParticleType(tn.decode('UTF-8'),
                                 charge = d['f3'][i],
                                 mass   = d['f4'][i])
            p = PointParticle( type_ = type_,
                               position = [xs[i],ys[i],zs[i]],
                               name = names[i].decode('UTF-8'),
                               beta = 1)
            self.children.append( p )
        
        atoms = self.children

        # minIdx = np.min(self.indices) - 1
        self.bonds = np.loadtxt("%s.bonds.txt" % prefix, dtype='i4') # - minIdx
        self.angles = np.loadtxt("%s.angles.txt" % prefix, dtype='i4') # - minIdx
        self.dihedrals = np.loadtxt("%s.dihedrals.txt" % prefix, dtype='i4')# - minIdx
        self.impropers = np.loadtxt("%s.impropers.txt" % prefix, dtype='i4')# - minIdx

    def generate(self):
        nt = Group(orientation = CanonicalNucleotideFactory.DefaultOrientation)
        nt.__dict__['sequence'] = self.sequence
        nt.__dict__['resname'] = self.resname              
        atoms = nt.children = [copy.copy(c) for c in self.children]
        for a in atoms:
            a.parent = nt
        nt.atoms_by_name = {a.name:a for a in nt.children}
        
        b = self.bonds
        a = self.angles
        d = self.dihedrals
        imp = self.impropers
        for i,j in zip(b[::2],b[1::2]):
            nt.add_bond( atoms[i], atoms[j], None )
        for i,j,k in zip(a[::3],a[1::3],a[2::3]):
            nt.add_angle( atoms[i], atoms[j], atoms[k], None )
        for i,j,k,l in zip(d[::4],d[1::4],d[2::4],d[3::4]):
            nt.add_dihedral( atoms[i], atoms[j], atoms[k], atoms[l], None )
        for i,j,k,l in zip(imp[::4],imp[1::4],imp[2::4],imp[3::4]):
            nt.add_improper( atoms[i], atoms[j], atoms[k], atoms[l], None )
        return nt

    
canonicalNtFwd = dict()
direction = 'fwd'
for seq in seqComplement.keys():
    for keystring,suff in zip(("5%s","%s","%s3","5%s3"),("-5prime","","-3prime","-singlet")):
        prefix = 'resources/%s-%s%s' % (seq,direction,suff)
        key = keystring % seq
        canonicalNtFwd[key] = CanonicalNucleotideFactory( prefix, seq )

canonicalNtRev = dict()
direction = 'rev'
for seq in seqComplement.keys():
    for keystring,suff in zip(("5%s","%s","%s3","5%s3"),("-5prime","","-3prime","-singlet")):
        prefix = 'resources/%s-%s%s' % (seq,direction,suff)
        key = keystring % seq
        canonicalNtRev[key] = CanonicalNucleotideFactory( prefix, seq )

with open("resources/enm-template-honeycomb.json") as ch:
    enmTemplateHC = json.load(ch)
with open("resources/enm-template-square.json") as ch:
    enmTemplateSQ = json.load(ch)
with open("resources/enm-corrections-honecomb.json") as ch:
    enmCorrectionsHC = json.load(ch)
